package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.rule.RuleClassifyDTO;
import com.gic.enterprise.dto.rule.RuleClassifySceneDTO;
import com.gic.enterprise.dto.rule.RuleDTO;
import com.gic.enterprise.entity.TabConfigRule;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.ConfigRuleApiService;
import com.gic.enterprise.service.ConfigRuleService;
import com.github.pagehelper.Page;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

@Service("configRuleApiService")
public class ConfigRuleApiServiceImpl implements ConfigRuleApiService {
    @Autowired
    private ConfigRuleService configRuleService;

    @Override
    public ServiceResponse<Integer> saveRule(RuleDTO dto) {
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(null, 1, dto.getName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        //填充
        dto.setParentRuleId(0);
        dto.setRuleLevel(1);
        Integer ruleId = configRuleService.saveRule(dto);
        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<Integer> editRule(RuleDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 1, dto.getName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        record.setName(dto.getName());
        record.setAppCode(dto.getAppCode());
        record.setAppName(dto.getAppName());
        configRuleService.update(record);
        return ServiceResponse.success(dto.getRuleId());
    }

    @Override
    public ServiceResponse<List<RuleDTO>> listRule(String search) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 1, null, 1, Integer.MAX_VALUE);
        if (page != null) {
            List<TabConfigRule> list = page.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return ServiceResponse.success(EntityUtil.changeEntityListNew(RuleDTO.class, list));
            }
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<com.gic.api.base.commons.Page<RuleClassifyDTO>> listRuleClassify(String search,
            Integer ruleId, Integer pageNum, Integer pageSize) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 2, ruleId, pageNum, pageSize);
        com.gic.api.base.commons.Page resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, RuleClassifyDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<com.gic.api.base.commons.Page<RuleClassifySceneDTO>> listRuleClassifyScene(String search,
            Integer ruleId, Integer pageNum, Integer pageSize) {
        Page<TabConfigRule> page = configRuleService.listRule(search, 3, ruleId, pageNum, pageSize);
        com.gic.api.base.commons.Page resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, RuleClassifyDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassify(RuleClassifyDTO dto) {
        ServiceResponse valid = validRuleClassify(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        dto.setRuleLevel(2);

        Integer ruleId = configRuleService.saveRuleClassify(dto);
        //更新操作ID链数据
        configRuleService.update(
                new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<String> editRuleClassify(RuleClassifyDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        ServiceResponse valid = validRuleClassify(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        record.setName(dto.getName());
        record.setClassifyIcon(dto.getClassifyIcon());
        record.setDesc(dto.getDesc());
        record.setRemark(dto.getRemark());
        configRuleService.update(record);
        return ServiceResponse.success(record.getName());
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassifyScene(RuleClassifySceneDTO dto) {
        ServiceResponse valid = validRuleClassifyScene(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }

        //上层分类
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());

        dto.setRuleLevel(3);
        Integer firstRuleId = record.getParentRuleId();
        Integer ruleId = configRuleService.saveRuleClassifyScene(dto);

        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId)
                .setRuleChain("_0_" + firstRuleId + "_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<String> editRuleClassifyScene(RuleClassifySceneDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，场景不存在");
        }
        if (record.getRuleLevel() != 3) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，不是场景类型");
        }
        ServiceResponse valid = validRuleClassifyScene(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        record.setRemark(dto.getRemark());
        record.setDesc(dto.getDesc());
        record.setMarketObject(dto.getMarketObject());
        record.setMarketScene(dto.getMarketScene());
        configRuleService.update(record);
        return ServiceResponse.success(record.getName());
    }

    @Override
    public ServiceResponse<String> deleteRule(Integer ruleId) {
        //规则不允许删除
//        TabConfigRule record = configRuleService.getByRuleId(ruleId);
//        if (record == null) {
//            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
//        }
//        if (record.getRuleLevel() != 1) {
//            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
//        }
//        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> deleteRuleClassify(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        //删除下面的场景
        List<TabConfigRule> list = configRuleService.listRule(null, 3, ruleId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabConfigRule tabConfigRule : list) {
                configRuleService.update(new TabConfigRule().setRuleId(tabConfigRule.getRuleId()).setStatus(0));
            }
        }
        return ServiceResponse.success(record.getName());
    }

    @Override
    public ServiceResponse<String> deleteRuleClassifyScene(Integer ruleId) {
        TabConfigRule record = configRuleService.getByRuleId(ruleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，场景不存在");
        }
        if (record.getRuleLevel() != 3) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景ID错误，不是场景类型");
        }
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setStatus(0));
        return ServiceResponse.success(record.getName());
    }

    private ServiceResponse<Void> validRuleClassify(RuleClassifyDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类名称不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 2, dto.getName(), dto.getParentRuleId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类名称重复");
        }
        if (StringUtils.isBlank(dto.getClassifyIcon())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类icon不能为空");
        }
        if (StringUtils.isBlank(dto.getDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类描述不能为空");
        }
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> validRuleClassifyScene(RuleClassifySceneDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，分类不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        if (StringUtils.isBlank(dto.getMarketScene())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景code不能为空");
        }
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景名称不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(dto.getRuleId(), 3, dto.getName(), dto.getParentRuleId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        if (StringUtils.isBlank(dto.getDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景描述不能为空");
        }
        Integer marketObject = dto.getMarketObject();
        boolean isNotRightMarketObject = marketObject == null
                || (marketObject != 1 && marketObject != 2 && marketObject != 3);
        if (isNotRightMarketObject) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "营销对象值非法");
        }
        return ServiceResponse.success();
    }
}
